from .multi import MultiDomainRankingMethodMapper
from .single import SingleDomainRankingMethodMapper

# isort: unique-list
__all__ = ["MultiDomainRankingMethodMapper", "SingleDomainRankingMethodMapper"]
